@@ -628,6 +628,8 @@ pyg_value_array_from_pyobject(GValue *value,
 
 	if (pspec && pspec->element_spec)
 	    type = G_PARAM_SPEC_VALUE_TYPE(pspec->element_spec);
+	else if (item == Py_None)
+	    type = G_TYPE_POINTER; /* store None as NULL */
 	else {
 	    type = pyg_type_from_object((PyObject *) item->ob_type);
 	    if (! type) {
@@ -847,7 +849,7 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
 
 	if (obj == Py_None)
 	    g_value_set_boxed(value, NULL);
-	if (G_VALUE_HOLDS(value, PY_TYPE_OBJECT))
+	else if (G_VALUE_HOLDS(value, PY_TYPE_OBJECT))
 	    g_value_set_boxed(value, obj);
 	else if (PyObject_TypeCheck(obj, &PyGBoxed_Type) &&
 		   G_VALUE_HOLDS(value, ((PyGBoxed *)obj)->gtype))
@@ -1328,13 +1330,18 @@ GClosure *
 gclosure_from_pyfunc(PyGObject *object, PyObject *func)
 {
     GSList *l;
-
-    for (l = object->closures; l; l = l->next) {
-	PyGClosure *pyclosure = l->data;
-	if (PyFunction_GetClosure(pyclosure->callback) ==
-		PyFunction_GetClosure(func)) {
-	    return (GClosure*)pyclosure;
-	}
+    PyGObjectData *inst_data;
+    inst_data = pyg_object_peek_inst_data(object->obj);
+    if (inst_data)
+    {
+        for (l = inst_data->closures; l; l = l->next) {
+            PyGClosure *pyclosure = l->data;
+            int res;
+            PyObject_Cmp(pyclosure->callback, func, &res);
+            if (!res) {
+                return (GClosure*)pyclosure;
+            }
+        }
     }
     return NULL;
 }
